#ifndef DUPDUMP_STORE_H
#define DUPDUMP_STORE_H 1

#if GCC_VERSION >= 40300
#define __hot		__attribute__((__hot__))
#define __cold		__attribute__((__cold__))
#else
#define __hot
#define __cold
#endif


#include <openssl/sha.h>

struct file_node
{
	char			*name;
	unsigned long long	size;
	unsigned char		sha1[2][SHA_DIGEST_LENGTH]; /* idx 0 is first1 is full */
	unsigned char		skip_compare:1;
	unsigned char		no_dup_possible:1;
	unsigned char		do_not_dump:1;
	unsigned char		unique:1;
	unsigned char		left:1;
	int			level;
	dev_t			dev;
	ino_t			ino;
	struct file_node	*next;
	struct file_node	*hash_next;
	struct dir_node		*parent;
	struct file_node	*duplicates;
	time_t			mtime;
	struct file_node	*ino_first; /* this will point the the first hardlink found */
};

struct dir_node
{
	char			*name;
	unsigned char		sha1[SHA_DIGEST_LENGTH];
	unsigned char		file_names_sha1[SHA_DIGEST_LENGTH];
	unsigned char		no_dup_possible:1;
	unsigned char		do_not_dump:1;
	unsigned char		left:1;
	int			level;
	dev_t			dev;
	ino_t			ino;
	struct dir_node		*subdirs;
	struct dir_node		*next_sibling; /* Links subdirs on the same level */
	struct file_node	*files;
	unsigned int		no_of_files;
	struct dir_node		*parent;
	struct dir_node		*hash_next; /* In the last phase, here we store duplicates TODO: rename to duplicate_next? */
	time_t			mtime;
	unsigned long long	total_size;
};


void			set_debug(const unsigned int level);
void			set_out(FILE *out);
void			dump_stats(void);
int			file_add(const char *file, const struct stat *s,
				const int level);
int			dir_add(const char *dir, const struct stat *s,
				const int level);
void			dump_files(void);
int			file_find_dups(void);
int			dir_find_dups(void);
struct dir_node		*dir_get_current(const unsigned int level);

void			dev_ino_seen_clean(void);

void			dump_dirs(void);

void			dump_duplicates(const unsigned long long min_size,
				const unsigned int zero);

#endif
